;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~|
; Filename: led_asm1.asm                                             |
;                                                                    |
; Author: David M. Alter, Texas Instruments Inc.                     |
;                                                                    |
; Last Modified: 10/24/00                                            |
;                                                                    |
; Description: This program cycles the LED's on the F2407 EVM board  |
; at a 250ms rate.  GP Timer2 is used to trigger a period match      |
; interrupt and provide the clocking for the LED updates.            |
;                                                                    |
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Address definitions
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        .include f2407.h

DAC0    .set    0000h           ;DAC0 input data reg (I/O space)
DAC1    .set    0001h           ;DAC1 input data reg (I/O space)
DAC2    .set    0002h           ;DAC2 input data reg (I/O space)
DAC3    .set    0003h           ;DAC3 input data reg (I/O space)
DACUD   .set    0004h           ;DAC update register (I/O space)
DIPSWCH .set    0008h           ;4 position DIP switch (I/O space)
LED     .set    000Ch           ;address of LED bank on EVM (I/O space)

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Constant definitions
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
LED_prd .set    58594           ;gives a 250ms LED update rate 
                                ;with a 1/128 timer prescaler @ 30MHz

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Uninitialized variable definitions
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        .bss    temp1,1         ;general purpose variable
        .bss    LED_index,1     ;LED index

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Interrupt vector table for core
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        .sect    "vectors"
        B       start           ;00h reset
int1:   B       int1            ;02h INT1
int2:   B       int2            ;04h INT2
int3:   B       gisr3           ;06h INT3
int4:   B       int4            ;08h INT4
int5:   B       int5            ;0Ah INT5
int6:   B       int6            ;0Ch INT6
int7:   B       int7            ;0Eh reserved
int8:   B       int8            ;10h INT8  user-defined
int9:   B       int9            ;12h INT9  user-defined
int10:  B       int10           ;14h INT10 user defined
int11:  B       int11           ;16h INT11 user defined
int12:  B       int12           ;18h INT12 user defined
int13:  B       int13           ;1Ah INT13 user defined
int14:  B       int14           ;1Ch INT14 user defined
int15:  B       int15           ;1Eh INT15 user defined
int16:  B       int16           ;20h INT16 user defined
int17:  B       int17           ;22h TRAP
int18:  B       int18           ;24h NMI
int19:  B       int19           ;26h reserved
int20:  B       int20           ;28h INT20 user defined
int21:  B       int21           ;2Ah INT21 user defined
int22:  B       int22           ;2Ch INT22 user defined
int23:  B       int23           ;2Eh INT23 user defined
int24:  B       int24           ;30h INT24 user defined
int25:  B       int25           ;32h INT25 user defined
int26:  B       int26           ;34h INT26 user defined
int27:  B       int27           ;36h INT27 user defined
int28:  B       int28           ;38h INT28 user defined
int29:  B       int29           ;3Ah INT29 user defined
int30:  B       int30           ;3Ch INT30 user defined
int31:  B       int31           ;3Eh INT31 user defined


**********************************************************************
*                     M A I N   R O U T I N E                        *
**********************************************************************

        .text
start:
;Disable global interrupts
;INTM=1 is reset state.  Can probably delete this for production.
        SETC    INTM            ;disable global interrupts

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Setup system control registers
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        LDP     #DP_PF1         ;set data page

        SPLK    #1000000111111101b, SCSR1
;                ||||||||||||||||
;                FEDCBA9876543210
* bit 15        1:      clear OSC FAIL FLAG
* bit 14        0:      CLKOUT = CPUCLK
* bit 13-12     00:     IDLE1 selected for low-power mode
* bit 11-9      000:    PLL x4 mode
* bit 8         1:      1 = reset DSP on OSC Fail detection
* bit 7         1:      1 = enable ADC module clock
* bit 6         1:      1 = enable SCI module clock
* bit 5         1:      1 = enable SPI module clock
* bit 4         1:      1 = enable CAN module clock
* bit 3         1:      1 = enable EVB module clock
* bit 2         1:      1 = enable EVA module clock
* bit 1         0:      reserved
* bit 0         1:      clear the ILLADR bit

        LACC    SCSR2
        OR      #0000000000000011b
        AND     #0000000000001111b
;                ||||||||||||||||
;                FEDCBA9876543210
* bit 15-6      0's:    reserved
* bit 5         0:      do NOT clear the WD OVERRIDE bit
* bit 4         0:      XMIF_HI-Z, 0=normal mode, 1=Hi-Z'd
* bit 3     no change   BOOT_EN pin
* bit 2     no change   MP/MC* pin
* bit 1-0      11:      11 = SARAM mapped to prog and data (default)

        SACL    SCSR2

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Disable the watchdog
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
kill_dog:
        LDP     #DP_PF1         ;set data page

        SPLK    #11101000b, WDCR
;                ||||||||
;                76543210
* bit 7         1:       clear WD flag
* bit 6         1:       disable the dog
* bit 5-3       101:     must be written as 101
* bit 2-0       000:     WDCLK divider = 1

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Set wait states for external memory interface on LF2407 EVM
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        LDP     #temp1          ;set data page

        SPLK    #0000000001000000b, temp1
;                ||||||||||||||||
;                FEDCBA9876543210
* bit 15-11     0's:    reserved
* bit 10-9      00:     bus visibility off
* bit 8-6       001:    1 wait-state for I/O space
* bit 5-3       000:    0 wait-state for data space
* bit 2-0       000:    0 wait state for program space

        OUT     temp1, WSGR

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Setup shared I/O pins
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        LDP     #DP_PF2         ;set data page

        SPLK    #0000000000000000b,MCRA
*                ||||||||||||||||
*                FEDCBA9876543210
* bit 15        0:      0=IOPB7,     1=TCLKINA
* bit 14        0:      0=IOPB6,     1=TDIRA
* bit 13        0:      0=IOPB5,     1=T2PWM/T2CMP
* bit 12        0:      0=IOPB4,     1=T1PWM/T1CMP
* bit 11        0:      0=IOPB3,     1=PWM6
* bit 10        0:      0=IOPB2,     1=PWM5
* bit 9         0:      0=IOPB1,     1=PWM4
* bit 8         0:      0=IOPB0,     1=PWM3
* bit 7         0:      0=IOPA7,     1=PWM2
* bit 6         0:      0=IOPA6,     1=PWM1
* bit 5         0:      0=IOPA5,     1=CAP3
* bit 4         0:      0=IOPA4,     1=CAP2/QEP2
* bit 3         0:      0=IOPA3,     1=CAP1/QEP1
* bit 2         0:      0=IOPA2,     1=XINT1
* bit 1         0:      0=IOPA1,     1=SCIRXD
* bit 0         0:      0=IOPA0,     1=SCITXD

        SPLK    #1111111000000000b,MCRB
*                ||||||||||||||||
*                FEDCBA9876543210
* bit 15        1:      0=reserved,  1=TMS2 (always write as 1)
* bit 14        1:      0=reserved,  1=TMS  (always write as 1)
* bit 13        1:      0=reserved,  1=TD0  (always write as 1)
* bit 12        1:      0=reserved,  1=TDI  (always write as 1)
* bit 11        1:      0=reserved,  1=TCK  (always write as 1)
* bit 10        1:      0=reserved,  1=EMU1 (always write as 1)
* bit 9         1:      0=reserved,  1=EMU0 (always write as 1)
* bit 8         0:      0=IOPD0,     1=XINT2/ADCSOC
* bit 7         0:      0=IOPC7,     1=CANRX
* bit 6         0:      0=IOPC6,     1=CANTX
* bit 5         0:      0=IOPC5,     1=SPISTE
* bit 4         0:      0=IOPC4,     1=SPICLK
* bit 3         0:      0=IOPC3,     1=SPISOMI
* bit 2         0:      0=IOPC2,     1=SPISIMO
* bit 1         0:      0=IOPC1,     1=BIO*
* bit 0         0:      0=IOPC0,     1=W/R*

        SPLK    #0000000000000000b,MCRC
*                ||||||||||||||||
*                FEDCBA9876543210
* bit 15        0:      reserved
* bit 14        0:      0=IOPF6,     1=IOPF6
* bit 13        0:      0=IOPF5,     1=TCLKINB
* bit 12        0:      0=IOPF4,     1=TDIRB
* bit 11        0:      0=IOPF3,     1=T4PWM/T4CMP
* bit 10        0:      0=IOPF2,     1=T3PWM/T3CMP
* bit 9         0:      0=IOPF1,     1=CAP6
* bit 8         0:      0=IOPF0,     1=CAP5/QEP4
* bit 7         0:      0=IOPE7,     1=CAP4/QEP3
* bit 6         0:      0=IOPE6,     1=PWM12
* bit 5         0:      0=IOPE5,     1=PWM11
* bit 4         0:      0=IOPE4,     1=PWM10
* bit 3         0:      0=IOPE3,     1=PWM9
* bit 2         0:      0=IOPE2,     1=PWM8
* bit 1         0:      0=IOPE1,     1=PWM7
* bit 0         0:      0=IOPE0,     1=CLKOUT

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Setup the core interrupts
;Note: This must be done before any peripheral interrupt is enabled.
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        LDP     #0h             ;set data page
        SPLK    #111111b,IFR    ;clear any pending core interrupts
        SPLK    #000100b,IMR    ;enable desired core interrupts

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Setup the Event Manager: GP Timer2
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        LDP     #DP_EVA          	;set data page
        SPLK    #0000h, T2CON   	;disable timer
        SPLK    #0000h, T2CNT   	;clear timer counter
        SPLK    #LED_prd, T2PR  	;set timer period

;Configure GPTCON register
        SPLK    #0000000000000000b, GPTCONA
*                ||||||||||||||||
*                FEDCBA9876543210
* bit 15        0:      reserved
* bit 14        0:      T2STAT, read-only
* bit 13        0:      T1STAT, read-only
* bit 12-11     00:     reserved
* bit 10-9      00:     no Timer 2 event starts ADC
* bit 8-7       00:     no Timer 1 event starts ADC
* bit 6         0:      all GP Timer compares disabled
* bit 5-4       00:     reserved
* bit 3-2       00:     Timer 2 compare forced low
* bit 1-0       00:     Timer 1 compare forced low

;Enable Timer2
      SPLK    #1101011101000000b, T2CON
*              ||||||||||||||||
*              FEDCBA9876543210
* bit 15-14     11:     emulator suspend has no effect
* bit 13        0:      reserved
* bit 12-11     10:     continuous up-count mode
* bit 10-8      111:    prescaler = x/128
* bit 7         0:      use own TENABLE bit
* bit 6         1:      enable timer
* bit 5-4       00:     internal clock source
* bit 3-2       00:     timer compare reload on underflow
* bit 1         0:      disable timer compare operation
* bit 0         0:      use own period register

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Setup the event manager interrupts
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        LDP     #DP_EVA          ;set data page
        SPLK    #0FFFFh, EVAIFRA ;clear all EVA group A interrupts
        SPLK    #0FFFFh, EVAIFRB ;clear all EVA group B interrupts
        SPLK    #0FFFFh, EVAIFRC ;clear all EVA group C interrupts
        SPLK    #00000h, EVAIMRA ;enabled desired EVA group A interrupts
        SPLK    #00001h, EVAIMRB ;enabled desired EVA group B interrupts
        SPLK    #00000h, EVAIMRC ;enabled desired EVA group C interrupts

        LDP     #DP_EVB          ;set data page
        SPLK    #0FFFFh, EVBIFRA ;clear all EVB group A interrupts
        SPLK    #0FFFFh, EVBIFRB ;clear all EVB group B interrupts
        SPLK    #0FFFFh, EVBIFRC ;clear all EVB group C interrupts
        SPLK    #00000h, EVBIMRA ;enabled desired EVB group A interrupts
        SPLK    #00000h, EVBIMRB ;enabled desired EVB group B interrupts
        SPLK    #00000h, EVBIMRC ;enabled desired EVB group C interrupts

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Other setup
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;LED index initialization
        LDP     #LED_index      ;set data page
        SPLK    #1h, LED_index  ;initialize the LED index

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Enable global interrupts
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        CLRC    INTM            ;enable global interrupts

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Main loop
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
loop:
        B       loop            ;branch to loop   



**********************************************************************
*  G E N E R A L  I N T E R R U P T  S E R V I C E  R O U T I N E S  *
**********************************************************************

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;General Interrupt Service Routine #3
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

STATUS3 .usect  "BLOCKB2", 2    ;must be located on data page 0
CONTEX3 .usect  "CONTEXT3", 2   ;locate anywhere in data space

gisr3:
;context save
        SST     #0, STATUS3      ;store ST0
        SST     #1, STATUS3      ;store ST1
        LDP     #CONTEX3         ;set data page
        SACH    CONTEX3          ;save ACCH
        SACL    CONTEX3+1        ;save ACCL

;clear the peripheral interrupt flag
        LDP     #DP_EVA          ;set data page
        SPLK    #00001h, EVAIFRB ;clear T2PINT flag

;update the LED's
        LDP     #LED_index       ;set data page
        OUT     LED_index, LED   ;light the LED
        LACC    LED_index,1      ;load LED index and shift left 1
        SACL    LED_index        ;store updated index
        SUB     #0010h           ;subtract the mask
        BCND    done, LT
        SPLK    #1h, LED_index   ;reset LED index to 1

;context restore
done:
        LDP     #CONTEX3         ;set data page
        LACL    CONTEX3+1        ;restore ACCL
        ADD     CONTEX3,16       ;restore ACCH
        LDP     #0               ;set data page
        LST     #1, STATUS3+1    ;restore ST1
        LST     #0, STATUS3      ;restore ST0
        CLRC    INTM             ;re-enable interrupts
        RET
